# Form implementation generated from reading ui file 'src/maestral_qt/resources/dropbox_location_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.2.3
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(688, 273)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setContentsMargins(-1, -1, -1, 0)
        self.gridLayout_2.setSpacing(20)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.pushButtonUnlink = QtWidgets.QPushButton(Dialog)
        self.pushButtonUnlink.setObjectName("pushButtonUnlink")
        self.gridLayout_2.addWidget(self.pushButtonUnlink, 3, 4, 1, 1)
        self.comboBoxPath = QtWidgets.QComboBox(Dialog)
        self.comboBoxPath.setObjectName("comboBoxPath")
        self.gridLayout_2.addWidget(self.comboBoxPath, 2, 1, 1, 6)
        self.pushButtonQuit = QtWidgets.QPushButton(Dialog)
        self.pushButtonQuit.setObjectName("pushButtonQuit")
        self.gridLayout_2.addWidget(self.pushButtonQuit, 3, 5, 1, 1)
        self.labelTitle = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelTitle.sizePolicy().hasHeightForWidth())
        self.labelTitle.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.labelTitle.setFont(font)
        self.labelTitle.setObjectName("labelTitle")
        self.gridLayout_2.addWidget(self.labelTitle, 0, 1, 1, 6)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem, 3, 0, 1, 1)
        self.pushButtonSelect = QtWidgets.QPushButton(Dialog)
        self.pushButtonSelect.setObjectName("pushButtonSelect")
        self.gridLayout_2.addWidget(self.pushButtonSelect, 3, 6, 1, 1)
        self.labelMessage = QtWidgets.QLabel(Dialog)
        self.labelMessage.setWordWrap(True)
        self.labelMessage.setObjectName("labelMessage")
        self.gridLayout_2.addWidget(self.labelMessage, 1, 1, 1, 6)
        self.labelIcon = QtWidgets.QLabel(Dialog)
        self.labelIcon.setMinimumSize(QtCore.QSize(60, 60))
        self.labelIcon.setMaximumSize(QtCore.QSize(60, 60))
        self.labelIcon.setText("")
        self.labelIcon.setObjectName("labelIcon")
        self.gridLayout_2.addWidget(self.labelIcon, 0, 0, 2, 1, QtCore.Qt.AlignmentFlag.AlignTop)
        self.spinner = QProgressIndicator(Dialog)
        self.spinner.setMinimumSize(QtCore.QSize(20, 20))
        self.spinner.setMaximumSize(QtCore.QSize(20, 20))
        self.spinner.setObjectName("spinner")
        self.gridLayout_2.addWidget(self.spinner, 3, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 3, 2, 1, 2)
        self.gridLayout.addLayout(self.gridLayout_2, 0, 1, 1, 6)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.comboBoxPath, self.pushButtonUnlink)
        Dialog.setTabOrder(self.pushButtonUnlink, self.pushButtonQuit)
        Dialog.setTabOrder(self.pushButtonQuit, self.pushButtonSelect)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        self.pushButtonUnlink.setText(_translate("Dialog", "Unlink"))
        self.pushButtonQuit.setText(_translate("Dialog", "Quit"))
        self.labelTitle.setText(_translate("Dialog", "Cannot find Dropbox folder"))
        self.pushButtonSelect.setText(_translate("Dialog", "Select"))
        self.labelMessage.setText(_translate("Dialog", "<html><head/><body><p>Your Dropbox folder has been moved or deleted from its original location. Syncing will not work until you move it back.</p><p>To move it back, click &quot;Quit&quot; below, move the Dropbox folder back to its original location, and launch Maestral again. To re-download your Dropbox, please select a new folder below.</p><p>Select &quot;Unlink&quot; to unlink your Dropbox account from Maestral.</p></body></html>"))
from maestral_qt.widgets import QProgressIndicator
