# Form implementation generated from reading ui file 'bandwidth_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.4.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_BandwidthDialog(object):
    def setupUi(self, BandwidthDialog):
        BandwidthDialog.setObjectName("BandwidthDialog")
        BandwidthDialog.resize(356, 255)
        self.gridLayout = QtWidgets.QGridLayout(BandwidthDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.updateButton = QtWidgets.QPushButton(parent=BandwidthDialog)
        self.updateButton.setObjectName("updateButton")
        self.gridLayout.addWidget(self.updateButton, 4, 2, 1, 1)
        self.cancelButton = QtWidgets.QPushButton(parent=BandwidthDialog)
        self.cancelButton.setObjectName("cancelButton")
        self.gridLayout.addWidget(self.cancelButton, 4, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem, 4, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(parent=BandwidthDialog)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridlayout = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridlayout.setObjectName("gridlayout")
        self.radioButtonUploadLimited = QtWidgets.QRadioButton(parent=self.groupBox_2)
        self.radioButtonUploadLimited.setObjectName("radioButtonUploadLimited")
        self.gridlayout.addWidget(self.radioButtonUploadLimited, 1, 0, 1, 1)
        self.numberInputUploadRate = QtWidgets.QDoubleSpinBox(parent=self.groupBox_2)
        self.numberInputUploadRate.setProperty("value", 1.0)
        self.numberInputUploadRate.setObjectName("numberInputUploadRate")
        self.gridlayout.addWidget(self.numberInputUploadRate, 1, 1, 1, 1)
        self.radioButtonUploadUnlimited = QtWidgets.QRadioButton(parent=self.groupBox_2)
        self.radioButtonUploadUnlimited.setObjectName("radioButtonUploadUnlimited")
        self.gridlayout.addWidget(self.radioButtonUploadUnlimited, 0, 0, 1, 1)
        self.label = QtWidgets.QLabel(parent=self.groupBox_2)
        self.label.setObjectName("label")
        self.gridlayout.addWidget(self.label, 1, 2, 1, 1)
        self.gridLayout.addWidget(self.groupBox_2, 2, 0, 1, 3)
        self.groupBox = QtWidgets.QGroupBox(parent=BandwidthDialog)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.radioButtonDownloadUnlimited = QtWidgets.QRadioButton(parent=self.groupBox)
        self.radioButtonDownloadUnlimited.setObjectName("radioButtonDownloadUnlimited")
        self.gridLayout_2.addWidget(self.radioButtonDownloadUnlimited, 1, 1, 1, 1)
        self.numberInputDownloadRate = QtWidgets.QDoubleSpinBox(parent=self.groupBox)
        self.numberInputDownloadRate.setProperty("value", 1.0)
        self.numberInputDownloadRate.setObjectName("numberInputDownloadRate")
        self.gridLayout_2.addWidget(self.numberInputDownloadRate, 2, 2, 1, 1)
        self.radioButtonDownloadLimited = QtWidgets.QRadioButton(parent=self.groupBox)
        self.radioButtonDownloadLimited.setObjectName("radioButtonDownloadLimited")
        self.gridLayout_2.addWidget(self.radioButtonDownloadLimited, 2, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.groupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 2, 3, 1, 1)
        self.gridLayout.addWidget(self.groupBox, 0, 0, 2, 3)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem1, 3, 0, 1, 3)

        self.retranslateUi(BandwidthDialog)
        QtCore.QMetaObject.connectSlotsByName(BandwidthDialog)

    def retranslateUi(self, BandwidthDialog):
        _translate = QtCore.QCoreApplication.translate
        BandwidthDialog.setWindowTitle(_translate("BandwidthDialog", "Bandwidth Dialog"))
        self.updateButton.setText(_translate("BandwidthDialog", "Update"))
        self.cancelButton.setText(_translate("BandwidthDialog", "Cancel"))
        self.groupBox_2.setTitle(_translate("BandwidthDialog", "Upload rate"))
        self.radioButtonUploadLimited.setText(_translate("BandwidthDialog", "Limit"))
        self.radioButtonUploadUnlimited.setText(_translate("BandwidthDialog", "Don\'t limit"))
        self.label.setText(_translate("BandwidthDialog", "MB/s"))
        self.groupBox.setTitle(_translate("BandwidthDialog", "Download rate"))
        self.radioButtonDownloadUnlimited.setText(_translate("BandwidthDialog", "Don\'t limit"))
        self.radioButtonDownloadLimited.setText(_translate("BandwidthDialog", "Limit"))
        self.label_3.setText(_translate("BandwidthDialog", "MB/s"))
